"""
django-hilbert is one of many Django apps which is a loose collection of utility functions.
It is a mixture of Python code and Javascript that I find myself writing over and 
over. Primarily it focuses around utilities for AJAX and testing.
"""

__version__ = '0.4.2'



import os
import sys

from django.conf import settings


if not settings.configured:
    settings.configure(
        DATABASES={
            'default': {
                'ENGINE': 'django.db.backends.sqlite3',
                'NAME': 'test.db',
            }
        },
        INSTALLED_APPS=(
            'django.contrib.auth',
            'django.contrib.contenttypes',
            'django.contrib.sessions',
            'django.contrib.sites',
            'django.contrib.messages',
            'hilbert',
        ),
        ROOT_URLCONF='hilbert.tests.urls',
        SITE_ID=1,
        TEST_RUNNER='hilbert.test.CoverageRunner',
        COVERAGE_MODULES=(
            'decorators',
            'http',
            'models',
            'views',
            'middleware',
        ),
        SSL_ENABLED=True
    )


from django.test.utils import get_runner


def runtests(*test_args):
    if not test_args:
        test_args = ['hilbert']
    parent = os.path.join(os.path.dirname(os.path.abspath(__file__)), "..", "..", )
    sys.path.insert(0, parent)
    TestRunner = get_runner(settings)
    test_runner = TestRunner(verbosity=1, interactive=True, failfast=False)
    failures = test_runner.run_tests(test_args)
    sys.exit(failures)


if __name__ == '__main__':
    runtests(*sys.argv[1:])




# -*- coding: utf-8 -*-
"""
Common test helper classes
"""

from inspect import ismodule
import random
import string

from django.conf import settings
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.db.models import get_app, get_apps
from django.test import TestCase as DjangoTestCase
from django.test.client import Client as DjangoClient
from django.test.simple import DjangoTestSuiteRunner


__all__ = (
    'Client',
    'TestCase',
    'CoverageRunner',
    'ViewTestMixin',
    'AuthViewMixin',
)


class Client(DjangoClient):

    def get(self, *args, **kwargs):
        is_ajax = kwargs.pop('is_ajax', False)
        if is_ajax:
            kwargs['HTTP_X_REQUESTED_WITH'] = 'XMLHttpRequest'
        return super(Client, self).get(*args, **kwargs)

    def post(self, *args, **kwargs):
        is_ajax = kwargs.pop('is_ajax', False)
        if is_ajax:
            kwargs['HTTP_X_REQUESTED_WITH'] = 'XMLHttpRequest'
        return super(Client, self).post(*args, **kwargs)    


class TestCase(DjangoTestCase):
    client_class = Client

    def get_random_string(self, length=10, choices=string.ascii_letters):
        return u''.join(random.choice(choices) for x in xrange(length))

    def get_random_email(self, domain=u'example.com'):
        local = self.get_random_string()
        return u'%s@%s' % (local, domain)

    def create_user(self, data=None):
        if data is None:
            data = {}
        defaults = {
            'username': self.get_random_string(),
            'email': self.get_random_email(),
            'password': self.get_random_string()
        }
        defaults.update(data)
        return User.objects.create_user(**defaults)


class CoverageRunner(DjangoTestSuiteRunner):
    """
    Coverage based test runner based on
    http://djangosnippets.org/snippets/2052/ and
    http://djangosnippets.org/snippets/705/
    """

    def get_coverage_modules(self, app_module):
        """
        Returns a list of modules to report coverage info for, given an
        application module.
        """

        app_path = app_module.__name__.split('.')[:-1]
        app_module = __import__('.'.join(app_path), globals(), locals(), app_path[-1])
        modules = [app_module]
        for module_name in settings.COVERAGE_MODULES:
            try:
                module = __import__('.'.join(app_path + [module_name]), {}, {}, module_name)
            except ImportError:
                module = None
            if ismodule(module):
                modules.append(module)
        return modules

    def run_tests(self, test_labels, extra_tests=None, **kwargs):
        run_with_coverage = hasattr(settings, 'COVERAGE_MODULES')

        # Set DEFAULT_TEST_LABELS to run just the tests we want
        # when we don't give 'test' any arguments
        if not test_labels:
            test_labels = getattr(settings, 'DEFAULT_TEST_LABELS', [])
        # If we've set DEFAULT_TEST_LABELS and we want to run them all,
        # just say 'test all'
        elif list(test_labels) == ['all']:
            test_labels = []

        if run_with_coverage:
            import coverage
            coverage.use_cache(0)
            coverage.start()

        result = super(CoverageRunner, self).run_tests(test_labels, extra_tests, **kwargs)

        if run_with_coverage:
            coverage.stop()
            coverage_modules = []
            if test_labels:
                for label in test_labels:
                    # Don't report coverage if you're only running a single
                    # test case.
                    if '.' not in label:
                        app = get_app(label)
                        coverage_modules.extend(self.get_coverage_modules(app))
            else:
                for app in get_apps():
                    coverage_modules.extend(self.get_coverage_modules(app))

            if coverage_modules:
                print ''
                print '----------------------------------------------------------------------'
                print ' Unit Test Code Coverage Results'
                print '----------------------------------------------------------------------'
                coverage.report(coverage_modules, show_missing=1)
                print '----------------------------------------------------------------------'

        return result


class ViewTestMixin(object):
    url_name = ''

    def setUp(self):
        super(ViewTestMixin, self).setUp()
        name, args, kwargs = self.get_url()
        self.url = reverse(name, args=args, kwargs=kwargs)

    def get_url(self):
        name = self.__class__.url_name
        args = self.get_url_args()
        kwargs = self.get_url_kwargs()
        return (name, args, kwargs)

    def get_url_args(self):
        return ()

    def get_url_kwargs(self):
        return {}

    def test_get_page(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)


class AuthViewMixin(ViewTestMixin):

    def setUp(self):
        super(AuthViewMixin, self).setUp()
        self.username = self.get_random_string()
        self.email = self.get_random_email()
        self.password = self.get_random_string()
        self.user = self.create_user(data={
            'username': self.username,
            'email': self.email,
            'password': self.password
        })
        self.client.login(username=self.username, password=self.password)

    def test_login_required(self):
        self.client.logout()
        response = self.client.get(self.url)
        redirect_url = '%s?next=%s' % (settings.LOGIN_URL, self.url)
        self.assertRedirects(response, redirect_url)



"""
Base test cases for Django-Hilbert.
"""

from django.conf import settings
from django.contrib.auth import models as auth

from hilbert.test import TestCase


class HilbertBaseTestCase(TestCase):
    urls = 'hilbert.tests.urls'
    username = 'hilbert'
    password = 'test'

    def setUp(self):
        super(HilbertBaseTestCase, self).setUp()
        self._ssl = getattr(settings, 'SSL_ENABLED', False)
        settings.SSL_ENABLED = True
        
    def tearDown(self):
        super(HilbertBaseTestCase, self).tearDown()
        settings.SSL_ENABLED = self._ssl

    def create_user(self, data=None):
        data = data or {}
        defaults = {
            'username': self.username,
            'password': self.password,
        }
        defaults.update(data)
        return super(HilbertBaseTestCase, self).create_user(defaults)



from django.conf.urls.defaults import *
from django.http import HttpResponse

from hilbert.decorators import ajax_login_required, ajax_only
from hilbert.decorators import anonymous_required, secure
from hilbert.http import JsonResponse


@ajax_login_required
def ajax_login_view(request):
    return HttpResponse()


@ajax_only
def ajax_only_view(request):
    return HttpResponse()


@anonymous_required
def anonymous_only_view(request):
    return HttpResponse()


@anonymous_required(url='/hilbert/test/simple/')
def anonymous_custom_view(request):
    return HttpResponse()


def json_response(request):
    return JsonResponse({'foo': 'bar'})


def simple_view(request):
    return HttpResponse()

@secure
def secure_view(request):
    return HttpResponse()


# Urls for testing
urlpatterns = patterns('',
    url(r'^hilbert/test/ajaxlogin/$', ajax_login_view),
    url(r'^hilbert/test/ajaxonly/$', ajax_only_view),
    url(r'^hilbert/test/jsonresponse/$', json_response),
    url(r'^hilbert/test/anonymous/$', anonymous_only_view),
    url(r'^hilbert/test/anonymous-custom/$', anonymous_custom_view),
    url(r'^hilbert/test/simple/$', simple_view),
    url(r'^hilbert/test/secure/$', secure_view),
    url(r'^$', simple_view),
)



from django.http import HttpResponse
from django.utils import simplejson


__all__ = (
    'JsonResponse',
)


class JsonResponse(HttpResponse):
    def __init__(self, content='', mimetype='application/javascript', status=None, content_type=None):
        content = simplejson.dumps(content)
        super(JsonResponse, self).__init__(content, mimetype, status, content_type)



"""
Tests for Django-Hilbert decorators.
"""

import os

from django.conf import settings

from hilbert.middleware import SSLRedirectMiddleware
from hilbert.tests.base import HilbertBaseTestCase


__all__ = (
    'AjaxLoginRequiredTestCase',
    'AjaxOnlyTestCase',
    'AnonymousRequiredTestCase',
    'SecureTestCase',
)


class AjaxLoginRequiredTestCase(HilbertBaseTestCase):

    def setUp(self):
        super(AjaxLoginRequiredTestCase, self).setUp()
        self.test_user = self.create_user()
        self.url = '/hilbert/test/ajaxlogin/'

    def test_basic_login_required(self):
        """
        Non-Ajax requests should still be handled with a login-required.
        """

        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)

        self.client.login(username=self.username, password=self.password)
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)

    def test_ajax_login_required(self):
        """
        Ajax requests should return custom headers to indicate login is required.
        """

        response = self.client.get(self.url, is_ajax=True)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(response['X-Django-Requires-Auth'])
        self.assertEqual(response['X-Django-Login-Url'], settings.LOGIN_URL)

        self.client.login(username=self.username, password=self.password)
        response = self.client.get(self.url, is_ajax=True)
        self.assertEqual(response.status_code, 200)
        self.assertTrue('X-Django-Requires-Auth' not in response)
        self.assertTrue('X-Django-Login-Url' not in response)


class AjaxOnlyTestCase(HilbertBaseTestCase):

    def setUp(self):
        super(AjaxOnlyTestCase, self).setUp()
        self.url = '/hilbert/test/ajaxonly/'

    def test_non_ajax_requests(self):
        """
        Non-Ajax requests will get a 400 - Bad Request.
        """

        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 400)

    def test_ajax_requests(self):
        """
        Ajax requests will get 200 status.
        """

        response = self.client.get(self.url, is_ajax=True)
        self.assertEqual(response.status_code, 200)


class AnonymousRequiredTestCase(HilbertBaseTestCase):

    def setUp(self):
        super(AnonymousRequiredTestCase, self).setUp()
        self.test_user = self.create_user()
        self.basic_url = '/hilbert/test/anonymous/'
        self.custom_url = '/hilbert/test/anonymous-custom/'
        self.custom_target = '/hilbert/test/simple/'

    def test_basic_anonymous_response(self):
        """
        Anonymous users will get the view response.
        """

        response = self.client.get(self.basic_url)
        self.assertEqual(response.status_code, 200)

    def test_custom_anonymous_response(self):
        """
        Anonymous users will get the view response.
        """

        response = self.client.get(self.custom_url)
        self.assertEqual(response.status_code, 200)

    def test_authenticated_response(self):
        """
        Authenticated users will get the redirect: '/' by default
        """

        self.client.login(username=self.username, password=self.password)
        response = self.client.get(self.basic_url)
        self.assertRedirects(response, '/')
        self.client.logout()

    def test_custom_authenticated_response(self):
        """
        Authenticated users will get the redirect url as specified
        """

        self.client.login(username=self.username, password=self.password)
        response = self.client.get(self.custom_url)
        self.assertRedirects(response, self.custom_target)
        self.client.logout()


class SecureTestCase(HilbertBaseTestCase):

    def setUp(self):
        super(SecureTestCase, self).setUp()
        self.url = '/hilbert/test/secure/'
        self.client.handler.load_middleware()
        self.middleware = SSLRedirectMiddleware()
        self.client.handler._request_middleware.insert(0, self.middleware.process_request)
        self.client.handler._view_middleware.insert(0, self.middleware.process_view)

    def test_response(self):
        """
        Non-secure requests will get a redirect.
        """

        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 301)

    def test_ssl_response(self):
        """
        Secure requests will get a successful response.
        """

        # Fake HTTPS
        # This might only work in Django 1.3
        response = self.client.get(self.url, **{'wsgi.url_scheme': 'https'})
        self.assertEqual(response.status_code, 200)

    def test_ssl_not_enabled(self):
        """
        Won't redirect if SSL is not marked as enabled.
        """
        
        ssl = getattr(settings, 'SSL_ENABLED', False)
        settings.SSL_ENABLED = False
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        settings.SSL_ENABLED = ssl




import re

from django.conf import settings
from django.http import HttpResponsePermanentRedirect, get_host


__all__ = (
    'SSLRedirectMiddleware',
    'SSLUserMiddleware',
)


SSL = 'SSL'


def _redirect(request, secure):
    protocol = secure and "https" or "http"
    newurl = "%s://%s%s" % (protocol, get_host(request), request.get_full_path())
    if getattr(settings, 'SSL_ENABLED', False):
        if settings.DEBUG and request.method == 'POST':
            raise RuntimeError("Django can't perform a redirect while maintaining POST data.")
        return HttpResponsePermanentRedirect(newurl)
    else:
        return None


class SSLRedirectMiddleware(object):
    """
    Pulls 'SSL' keyword out of url pattern definition or matches patterns in
    SSL_PATTERNS in the settings to determine if this view should be forced onto SSL.

    Based on http://djangosnippets.org/snippets/85/ and
    http://djangosnippets.org/snippets/880/.
    """

    def process_request(self, request):
        # Check settings patterns
        urls = tuple([re.compile(url) for url in getattr(settings, 'SSL_PATTERNS', [])])
        secure = any([url.search(request.path) for url in urls])
        if request.is_secure():
            if secure:
                request.keep_secure = True
        else:
            if secure:
               return _redirect(request, True) 
   
    def process_view(self, request, view_func, view_args, view_kwargs):
        # Check kwargs
        if SSL in view_kwargs:
            secure = view_kwargs[SSL]
            del view_kwargs[SSL]
        else:
            secure = False
        if request.is_secure():
            if not secure and not getattr(request, 'keep_secure', False):
                if getattr(settings, 'SSL_WHITELIST', False):
                    # Redirect off SSL
                    return _redirect(request, False)
        else:
            if secure:
               return _redirect(request, True)               


class SSLUserMiddleware(object):
    """
    Ensures that all requests for authenticated users are done over SSL.
    """

    def process_request(self, request):
        user = getattr(request, 'user', None)
        if user and user.is_authenticated():
            if request.is_secure():
                request.keep_secure = True
            else:
                return _redirect(request, True)




from functools import partial, wraps

from django import http
from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.shortcuts import redirect
from django.utils.decorators import available_attrs

from hilbert.middleware import _redirect


__all__ = (
    'ajax_login_required',
    'ajax_only',
    'anonymous_required',
    'secure',
)


def ajax_login_required(view_func):
    """Handle non-authenticated users differently if it is an AJAX request."""

    @wraps(view_func, assigned=available_attrs(view_func))
    def _wrapped_view(request, *args, **kwargs):
        if request.is_ajax():
            if request.user.is_authenticated():
                return view_func(request, *args, **kwargs)
            else:
                response = http.HttpResponse()
                response['X-Django-Requires-Auth'] = True
                response['X-Django-Login-Url'] = settings.LOGIN_URL
                return response
        else:
            return login_required(view_func)(request, *args, **kwargs)
    return _wrapped_view


def ajax_only(view_func):
    """Required the view is only accessed via AJAX."""

    @wraps(view_func, assigned=available_attrs(view_func))
    def _wrapped_view(request, *args, **kwargs):
        if request.is_ajax():
            return view_func(request, *args, **kwargs)
        else:
            return http.HttpResponseBadRequest()
    return _wrapped_view


def anonymous_required(func=None, url=None):
    """Required that the user is not logged in."""

    url = url or "/"

    def _dec(view_func):
        @wraps(view_func, assigned=available_attrs(view_func))
        def _wrapped_view(request, *args, **kwargs):
            if request.user.is_authenticated():
                return redirect(url)
            else:
                return view_func(request, *args, **kwargs)
        return _wrapped_view

    if func is None:
        return _dec
    else:
        return _dec(func)


def secure(view_func):
    """Handles SSL redirect on the view level."""

    @wraps(view_func, assigned=available_attrs(view_func))
    def _wrapped_view(request, *args, **kwargs):
        if not request.is_secure():
            redirect = _redirect(request, True)
            if redirect:
                # Redirect might be None if SSL is not enabled
                return redirect
        return view_func(request, *args, **kwargs)
    return _wrapped_view




"""
Tests for Django-Hilbert http responses.
"""

from django.utils import simplejson

from hilbert.tests.base import HilbertBaseTestCase


__all__ = (
    'JsonResponseTestCase',
)


class JsonResponseTestCase(HilbertBaseTestCase):

    def setUp(self):
        super(JsonResponseTestCase, self).setUp()
        self.url = '/hilbert/test/jsonresponse/'

    def test_basic_response(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)

    def test_encoding(self):
        response = self.client.get(self.url)
        content = simplejson.loads(response.content)
        self.assertEqual(content, {'foo': 'bar'})

    def test_mimetype(self):
        response = self.client.get(self.url)
        self.assertEqual(response['Content-Type'], 'application/javascript')



import os
from setuptools import setup, find_packages


def read_file(filename):
    """Read a file into a string"""
    path = os.path.abspath(os.path.dirname(__file__))
    filepath = os.path.join(path, filename)
    try:
        return open(filepath).read()
    except IOError:
        return ''


setup(
    name='django-hilbert',
    version=__import__('hilbert').__version__,
    author='Mark Lavin',
    author_email='markdlavin@gmail.com',
    packages=find_packages(exclude=['example']),
    include_package_data=True,
    url='https://github.com/mlavin/django-hilbert',
    license='LICENSE.txt',
    description=u' '.join(__import__('hilbert').__doc__.splitlines()).strip(),
    classifiers=[
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
    ],
    long_description=read_file('README.rst'),
    zip_safe=False, # because we're including media that Django needs
)
